import {MeterbusLibUtils} from './utils'


export namespace MeterbusLibCodeTables { 
    export const MEDIUM_CODE : string[] =
            [
                'Other', 
                'Oil', 
                'Electrity', 
                'Gas', 
                'Heat (Volume measured at return temperature: outlet',
                'Steam', 
                'Hot  Water', 
                'Water', 
                'Heat Cost Allocator', 
                'Compressed Air',
                'Cooling Load Meter (Volume measured at return temperature: outlet', 
                'Cooling Load Meter (Volume measured at flow temperature: inlet',
                'Heat (Volume measured at flow temperature: inlet)', 
                'Heat / Cooling Load Meter',
                'Bus / System', 
                'Unknown Medium',
                'Reserved (10)',
                'Reserved (11)',
                'Reserved (12)',
                'Reserved (13)',
                'Reserved (14)',
                'Reserved (15)',
                'Cold Water',
                'Dual Water',
                'Pressure',
                'A/D Converter',
            ]

    export const DIF_FUNCTION_FIELD : string[] =
            [
                'Instantaneous value',
                'Maximum value',
                'Minimum value',
                'Value during error state'
            ]

    export class DCFt { 
        name : string
        length : number
        encoder : (v: number[]) => any
        constructor(n : string, l : number, e : (v: number[]) => any) {
            this.name = n
            this.length = l
            this.encoder = e
        }
    }

    export const DIF_CODING_FIELD : DCFt[] =
            [
                new DCFt('No Data',                   0, (x)=>{ return x}),
                new DCFt('8 Bit Integer',             1, (x)=>{ return x[0]}),
                new DCFt('16 Bit Integer',            2, (x)=>{ return (x[1] << 8) + x[0]}),
                new DCFt('24 Bit Integer',            3, (x)=>{ return (x[2] << 16) + (x[1] << 8) + x[0]}),
                new DCFt('32 Bit Integer',            4, (x)=>{ return (x[3] << 24) + (x[2] << 16) + (x[1] << 8) + x[0]}),
                new DCFt('32 Bit Real',               4, (x)=>{ return x}), // FIXME
                new DCFt('48 Bit Integer',            6, (x)=>{ return (x[5] << 40) + (x[4] << 32) + (x[3] << 24) + (x[2] << 16) + (x[1] << 8) + x[0]}),       
                new DCFt('64 Bit Integer',            8, (x)=>{ return (x[7] << 56) + (x[6] << 48) + (x[5] << 40) + (x[4] << 32) + (x[3] << 24) + (x[2] << 16) + (x[1] << 8) + x[0]}),
                new DCFt('Selection for Readout',     0, (x)=>{ return x}),
                new DCFt('2 Digit BCD',               1, (x)=>{ return MeterbusLibUtils.bcd(x)}),
                new DCFt('4 Digit BCD',               2, (x)=>{ return MeterbusLibUtils.bcd(x)}),
                new DCFt('6 Digit BCD',               3, (x)=>{ return MeterbusLibUtils.bcd(x)}),
                new DCFt('8 Digit BCD',               4, (x)=>{ return MeterbusLibUtils.bcd(x)}),
                new DCFt('variable length',          -1, (x)=>{ return x}),
                new DCFt('12 Digit BCD',              6, (x)=>{ return MeterbusLibUtils.bcd(x)}),
                new DCFt('Special Function',          0, (x)=>{ return x})
            ]


}