import {MeterbusLibUtils} from './utils'

import * as mocha from 'mocha'
import * as chai from 'chai'

const expect = chai.expect


describe('The Meterbus Library Utils', () => {
    it('should convert a bcd to a number (10203)', () => {
        expect(MeterbusLibUtils.bcd([1, 2, 3])).to.equal(10203)
    })
    it('should convert a bcd to a number (0010203)', () => {
        expect(MeterbusLibUtils.bcd([0, 1, 2, 3])).to.equal(10203)
    })
    it('should convert a bcd to a number (0)', () => {
        expect(MeterbusLibUtils.bcd([0, 0, 0])).to.equal(0)
    })
    it('should convert a bcd to a number (0)', () => {
        expect(MeterbusLibUtils.bcd([0, 0, 0, 0])).to.equal(0)
    })
    it('should convert a bcd to a number (99999999)', () => {
        expect(MeterbusLibUtils.bcd([0x99, 0x99, 0x99, 0x99])).to.equal(99999999)
    })
    it('should convert two number to manufacturer code', () => {
        expect(MeterbusLibUtils.manufCode([0x2e, 0x19])).to.equal("FIN")
    })
})

describe('The jsonPrepare function in the Meterbus Library Utils', () => {
    it('should forward regular properties of an object', () => {
        let objIn = {'a': 1, 'b':2}
        let objOut = objIn
        expect(MeterbusLibUtils.jsonPrepaper(objIn, [])).to.deep.equal(objOut)
    })
    it('should hide properties in the hide list', () => {
        let objIn = {'a': 1, 'b':2}
        let objOut = {'a': 1}
        expect(MeterbusLibUtils.jsonPrepaper(objIn, ['b'])).to.deep.equal(objOut)
    })
    it('should hide properties beginning with two underscores', () => {
        let objIn = {'a': 1, '__b':2}
        let objOut = {'a': 1}
        expect(MeterbusLibUtils.jsonPrepaper(objIn, [])).to.deep.equal(objOut)
    })
    it('should remove one leading underscore from property names', () => {
        let objIn = {'_a': 1, 'b':2}
        let objOut = {'a': 1, 'b':2}
        expect(MeterbusLibUtils.jsonPrepaper(objIn, [])).to.deep.equal(objOut)
    })
})

describe('The clone function in the Meterbus Library Utils', () => {
    it('should clone the attributes of an object', () => {
        let objIn = {'a': 1, 'b': 2}
        let objOut = {'a': 1, 'b': 2}
        expect(MeterbusLibUtils.clone(objIn)).to.deep.equal(objOut)
    })
    it('should not just return the same object', () => {
        let objIn = {'a': 1, 'b':2}
        let objOut = objIn
        expect(MeterbusLibUtils.clone(objIn)).to.not.equal(objOut)
    })
})